#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#define STEP 100000000

/* cosmic.c -- a cosmic ray detector
 * by the club Inutile -- www.inutile.ens.fr
 * license: iGNUtile LPG v3 -- http://www.inutile.ens.fr/lpg/ */

int main(int argc, char **argv)
{
  if (argc != 3) {
    fprintf(stderr, "Usage: %s SIZE DURATION\n", argv[0]);
    return 1;
  }

  long ram = atol(argv[1]);
  int dur = atoi(argv[2]);

  if (!ram) {
    fprintf(stderr,
        "%s: FATAL: argument 1 must be number of bytes to allocate (got %s)",
        argv[0], argv[1]);
    return 1;
  }
  if (!dur) {
    fprintf(stderr,
        "%s: FATAL: argument 2 must be a number of seconds to wait (got %s)",
        argv[0], argv[2]);
    return 1;
  }

  char* mem;
  mem = (char*) malloc(ram);
  if (!mem) {
    fprintf(stderr,
        "%s: FATAL: could not allocate %ld bytes",
        argv[0], ram);
    return 2;
  }

  int seed, t;
  long i, detected;
  char expected;

  while(1) {
    seed = time(NULL);
    srandom(seed);
    /* set ram */
    for (i = 0; i < ram; i++) {
      mem[i] = random() & 0xff;
      if (!(i % STEP))
        fprintf(stderr, "%s: wrote %ld of %ld bytes\n", argv[0], i, ram);
    }
    /* wait */
    sleep(dur);
    t = seed + dur;
    srandom(seed);
    /* check ram */
    detected = 0;
    for (i = 0; i < ram; i++) {
      if (mem[i] != (expected = random() & 0xff)) {
        detected++;
        printf("%s: [%d] detected ray at 0x%.8lx: expected %.2x, read %.2x\n",
            argv[0], t, i, expected, mem[i]);
      }
      if (!(i % STEP))
        fprintf(stderr, "%s: checked %ld of %ld bytes\n", argv[0], i, ram);
    }
    if (!detected) {
      printf("%s: [%d] no cosmic rays detected\n", argv[0], t);
    } else {
      printf("%s: [%d] %ld cosmic ray%s detected\n", argv[0], t,
          detected, detected == 1 ? "" : "s");
    }
  }

  free((void*) mem);

  return 0;
}

